@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ $widget = (new CWidget())
 	->setControls((new CTag('nav', true,
 		(new CList())
 			->addItem(new CRedirectButton(_('Create media type'), 'zabbix.php?action=mediatype.edit'))
+			->addItem(new CRedirectButton(_('Import'), 'conf.import.php?rules_preset=mediatype'))
 		))
 			->setAttribute('aria-label', _('Content controls'))
 	)
@@ -61,11 +62,12 @@ $mediaTypeTable = (new CTableInfo())
 			(new CCheckBox('all_media_types'))
 				->onClick("checkAll('".$mediaTypeForm->getName()."', 'all_media_types', 'mediatypeids');")
 		))->addClass(ZBX_STYLE_CELL_WIDTH),
-		make_sorting_header(_('Name'), 'description', $data['sort'], $data['sortorder']),
+		make_sorting_header(_('Name'), 'name', $data['sort'], $data['sortorder']),
 		make_sorting_header(_('Type'), 'type', $data['sort'], $data['sortorder']),
 		_('Status'),
 		_('Used in actions'),
-		_('Details')
+		_('Details'),
+		_('Action')
 	]);
 
 foreach ($data['mediatypes'] as $mediaType) {
@@ -85,14 +87,6 @@ foreach ($data['mediatypes'] as $mediaType) {
 			$details = _('GSM modem').NAME_DELIMITER.'"'.$mediaType['gsm_modem'].'"';
 			break;
 
-		case MEDIA_TYPE_JABBER:
-			$details = _('Jabber identifier').NAME_DELIMITER.'"'.$mediaType['username'].'"';
-			break;
-
-		case MEDIA_TYPE_EZ_TEXTING:
-			$details = _('Username').NAME_DELIMITER.'"'.$mediaType['username'].'"';
-			break;
-
 		default:
 			$details = '';
 			break;
@@ -130,7 +124,14 @@ foreach ($data['mediatypes'] as $mediaType) {
 			->addClass(ZBX_STYLE_RED)
 			->addSID();
 
-	$name = new CLink($mediaType['description'], '?action=mediatype.edit&mediatypeid='.$mediaType['mediatypeid']);
+	$test_link = (new CButton('mediatypetest_edit', _('Test')))
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->setEnabled(MEDIA_TYPE_STATUS_ACTIVE == $mediaType['status'])
+		->onClick('return PopUp("popup.mediatypetest.edit",'.CJs::encodeJson([
+			'mediatypeid' => $mediaType['mediatypeid']
+		]).', "mediatypetest_edit", this);');
+
+	$name = new CLink($mediaType['name'], '?action=mediatype.edit&mediatypeid='.$mediaType['mediatypeid']);
 
 	// append row
 	$mediaTypeTable->addRow([
@@ -139,7 +140,8 @@ foreach ($data['mediatypes'] as $mediaType) {
 		media_type2str($mediaType['typeid']),
 		$status,
 		$actionColumn,
-		$details
+		$details,
+		$test_link
 	]);
 }
 
@@ -150,6 +152,14 @@ $mediaTypeForm->addItem([
 	new CActionButtonList('action', 'mediatypeids', [
 		'mediatype.enable' => ['name' => _('Enable'), 'confirm' => _('Enable selected media types?')],
 		'mediatype.disable' => ['name' => _('Disable'), 'confirm' => _('Disable selected media types?')],
+		'mediatype.export' => ['name' => _('Export'), 'redirect' =>
+			(new CUrl('zabbix.php'))
+				->setArgument('action', 'export.mediatypes.xml')
+				->setArgument('backurl', (new CUrl('zabbix.php'))
+					->setArgument('action', 'mediatype.list')
+					->getUrl())
+				->getUrl()
+		],
 		'mediatype.delete' => ['name' => _('Delete'), 'confirm' => _('Delete selected media types?')]
 	], 'mediatype')
 ]);
