@@ -37,10 +37,15 @@ class CHostImporter extends CImporter {
 		$hostsToCreate = [];
 		$hostsToUpdate = [];
 		$templateLinkage = [];
+		$tmpls_to_clear = [];
 
 		foreach ($hosts as $host) {
-			// preserve host related templates to massAdd them later
-			if ($this->options['templateLinkage']['createMissing'] && !empty($host['templates'])) {
+			/*
+			 * Save linked templates for 2 purposes:
+			 *  - save linkages to add in case if 'create new' linkages is checked;
+			 *  - calculate missing linkages in case if 'delete missing' is checked.
+			 */
+			if (!empty($host['templates'])) {
 				foreach ($host['templates'] as $template) {
 					$templateId = $this->referencer->resolveTemplate($template['name']);
 					if (!$templateId) {
@@ -66,6 +71,33 @@ class CHostImporter extends CImporter {
 			}
 		}
 
+		// Get template linkages to unlink and clear.
+		if ($hostsToUpdate && $this->options['templateLinkage']['deleteMissing']) {
+			// Get already linked templates.
+			$db_template_links = API::Host()->get([
+				'output' => ['hostids'],
+				'selectParentTemplates' => ['hostid'],
+				'hostids' => zbx_objectValues($hostsToUpdate, 'hostid'),
+				'preservekeys' => true
+			]);
+
+			foreach ($db_template_links as &$db_template_link) {
+				$db_template_link = zbx_objectValues($db_template_link['parentTemplates'], 'templateid');
+			}
+			unset($db_template_link);
+
+			foreach ($hostsToUpdate as $host) {
+				if (array_key_exists($host['host'], $templateLinkage)) {
+					$tmpls_to_clear[$host['hostid']] = array_diff($db_template_links[$host['hostid']],
+						zbx_objectValues($templateLinkage[$host['host']], 'templateid')
+					);
+				}
+				else {
+					$tmpls_to_clear[$host['hostid']] = $db_template_links[$host['hostid']];
+				}
+			}
+		}
+
 		// create/update hosts
 		if ($this->options['hosts']['createMissing'] && $hostsToCreate) {
 			$newHostIds = API::Host()->create($hostsToCreate);
@@ -75,7 +107,7 @@ class CHostImporter extends CImporter {
 
 				$this->referencer->addHostRef($hostHost, $hostId);
 
-				if (!empty($templateLinkage[$hostHost])) {
+				if ($this->options['templateLinkage']['createMissing'] && !empty($templateLinkage[$hostHost])) {
 					API::Template()->massAdd([
 						'hosts' => ['hostid' => $hostId],
 						'templates' => $templateLinkage[$hostHost]
@@ -94,7 +126,16 @@ class CHostImporter extends CImporter {
 			foreach ($hostsToUpdate as $host) {
 				$this->processedHostIds[$host['host']] = $host['hostid'];
 
-				if (!empty($templateLinkage[$host['host']])) {
+				// Drop existing template linkages if 'delete missing' selected.
+				if (array_key_exists($host['hostid'], $tmpls_to_clear) && $tmpls_to_clear[$host['hostid']]) {
+					API::Host()->massRemove([
+						'hostids' => [$host['hostid']],
+						'templateids_clear' => $tmpls_to_clear[$host['hostid']]
+					]);
+				}
+
+				// Make new template linkages.
+				if ($this->options['templateLinkage']['createMissing'] && !empty($templateLinkage[$host['host']])) {
 					API::Template()->massAdd([
 						'hosts' => $host,
 						'templates' => $templateLinkage[$host['host']]
