@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,6 +25,11 @@ class CControllerResponseData extends CControllerResponse {
 	private $title = null;
 	private $file_name = null;
 
+	/**
+	 * @var bool $view_enabled  true - send view and layout; false - send layout only.
+	 */
+	private $view_enabled = true;
+
 	public function __construct($data) {
 		$this->data = $data;
 	}
@@ -48,4 +53,22 @@ class CControllerResponseData extends CControllerResponse {
 	public function getFileName() {
 		return $this->file_name;
 	}
+
+	/**
+	 * Prohibits sending view.
+	 */
+	public function disableView() {
+		$this->view_enabled = false;
+
+		return $this;
+	}
+
+	/**
+	 * Returns current value of view_enabled variable.
+	 *
+	 * @return bool
+	 */
+	public function isViewEnabled() {
+		return $this->view_enabled;
+	}
 }
