@@ -759,8 +759,13 @@ function makePageFooter($with_version = true) {
  * @return CComboBox
  */
 function makeAdministrationGeneralMenu($selected) {
+	$autoreg_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'autoreg.edit')
+		->getUrl();
+
 	return new CComboBox('configDropDown', $selected, 'redirect(this.options[this.selectedIndex].value);', [
 		'adm.gui.php' => _('GUI'),
+		$autoreg_url => _('Auto registration'),
 		'adm.housekeeper.php' => _('Housekeeping'),
 		'adm.images.php' => _('Images'),
 		'adm.iconmapping.php' => _('Icon mapping'),
@@ -872,7 +877,9 @@ function makeActionIcon(array $icon_data) {
 	}
 
 	if (array_key_exists('hint', $icon_data)) {
-		$icon->setHint($icon_data['hint'], '', true, 'max-width: '.ZBX_ACTIONS_POPUP_MAX_WIDTH.'px;');
+		$icon
+			->addClass(ZBX_STYLE_CURSOR_POINTER)
+			->setHint($icon_data['hint'], '', true, 'max-width: '.ZBX_ACTIONS_POPUP_MAX_WIDTH.'px;');
 	}
 	elseif (array_key_exists('title', $icon_data)) {
 		$icon->setTitle($icon_data['title']);
@@ -887,6 +894,20 @@ function makeActionIcon(array $icon_data) {
 	return $icon;
 }
 
+/**
+ * Renders an icon for a description.
+ *
+ * @param string $description
+ *
+ * @return CSpan
+ */
+function makeDescriptionIcon($description) {
+	return (new CSpan())
+		->addClass(ZBX_STYLE_ICON_DESCRIPTION)
+		->addClass(ZBX_STYLE_CURSOR_POINTER)
+		->setHint(zbx_str2links($description), 'hintbox-description');
+}
+
 /**
  * Renders an error icon like red [i] with error message
  *
