@@ -250,7 +250,8 @@ function zbx_construct_menu(&$main_menu, &$sub_menus, &$page, $action = null) {
 						'adm.triggerseverities.php',
 						'adm.triggerdisplayoptions.php',
 						'adm.other.php'
-					]
+					],
+					'active_if' => ['autoreg.edit']
 				],
 				[
 					'url' => 'zabbix.php',
@@ -269,7 +270,9 @@ function zbx_construct_menu(&$main_menu, &$sub_menus, &$page, $action = null) {
 					'label' => _('User groups')
 				],
 				[
-					'url' => 'users.php',
+					'url' => 'zabbix.php',
+					'action' => 'user.list',
+					'active_if' => ['user.edit', 'user.list'],
 					'label' => _('Users')
 				],
 				[
@@ -297,7 +300,7 @@ function zbx_construct_menu(&$main_menu, &$sub_menus, &$page, $action = null) {
 			'pages' => [
 				[
 					'url' => 'index.php',
-					'sub_pages' => ['profile.php']
+					'sub_pages' => ['zabbix.php']
 				]
 			]
 		]
