@@ -180,7 +180,7 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 			}
 		}
 	}
-	$pageHeader->addCssFile('styles/'.CHtml::encode($theme).'.css');
+	$pageHeader->addCssFile('assets/styles/'.CHtml::encode($theme).'.css');
 
 	if ($page['file'] == 'sysmap.php') {
 		$pageHeader->addCssFile('imgstore.php?css=1&output=css');
@@ -198,7 +198,7 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 		$pageHeader->addJsFile((new CUrl('jsLoader.php'))
 			->setArgument('ver', ZABBIX_VERSION)
 			->setArgument('lang', CWebUser::$data['lang'])
-			->setArgument('showGuiMessaging', $is_standard_page ? 1 : null)
+			->setArgument('showGuiMessaging', ($is_standard_page && !CWebUser::isGuest()) ? 1 : null)
 			->getUrl()
 		);
 
@@ -215,7 +215,7 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 	$pageHeader->display();
 ?>
 <body lang="<?= CWebUser::getLang() ?>">
-<output class="<?= ZBX_STYLE_MSG_BAD_GLOBAL ?>" id="msg-bad-global"></output>
+<output class="<?= ZBX_STYLE_MSG_GLOBAL_FOOTER.' '.ZBX_STYLE_MSG_WARNING ?>" id="msg-global-footer"></output>
 <?php
 }
 
@@ -232,7 +232,7 @@ $message_error = null;
 $messages = [];
 
 // this code show messages generated by MVC pages
-if (CSession::keyExists('messageOk') || CSession::keyExists('messageError')) {
+if ($page['type'] == PAGE_TYPE_HTML && (CSession::keyExists('messageOk') || CSession::keyExists('messageError'))) {
 	if (CSession::keyExists('messages')) {
 		$messages = CSession::getValue('messages');
 		CSession::unsetValue(['messages']);
@@ -276,10 +276,6 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 // unset multiple variables
 unset($table, $top_page_row, $menu_table, $main_menu_row, $sub_menu_table, $sub_menu_rows);
 
-if ($page['type'] == PAGE_TYPE_HTML && $is_standard_page) {
-	zbx_add_post_js('initMessages();');
-}
-
 // if a user logs in after several unsuccessful attempts, display a warning
 if ($failedAttempts = CProfile::get('web.login.attempt.failed', 0)) {
 	$attempip = CProfile::get('web.login.attempt.ip', '');
