@@ -23,8 +23,8 @@ require_once dirname(__FILE__).'/js/configuration.triggers.edit.js.php';
 
 $widget = (new CWidget())->setTitle(_('Triggers'));
 
-// append host summary to widget header
-if (!empty($data['hostid'])) {
+// Append host summary to widget header.
+if ($data['hostid'] != 0) {
 	$widget->addItem(get_header_host_table('triggers', $data['hostid']));
 }
 
@@ -55,6 +55,7 @@ $readonly = ($data['limited'] || $discovered_trigger);
 
 if ($readonly) {
 	$triggersForm
+		->addVar('opdata', $data['opdata'])
 		->addVar('recovery_mode', $data['recovery_mode'])
 		->addVar('type', $data['type'])
 		->addVar('correlation_mode', $data['correlation_mode'])
@@ -69,17 +70,25 @@ if (!empty($data['templates'])) {
 
 if ($discovered_trigger) {
 	$triggersFormList->addRow(_('Discovered by'), new CLink($data['discoveryRule']['name'],
-		'trigger_prototypes.php?parent_discoveryid='.$data['discoveryRule']['itemid']
+		(new CUrl('trigger_prototypes.php'))
+			->setArgument('form', 'update')
+			->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
+			->setArgument('triggerid', $data['triggerDiscovery']['parent_triggerid'])
 	));
 }
 
-$triggersFormList->addRow(
-	(new CLabel(_('Name'), 'description'))->setAsteriskMark(),
-	(new CTextBox('description', $data['description'], $readonly))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		->setAriaRequired()
-		->setAttribute('autofocus', 'autofocus')
-);
+$triggersFormList
+	->addRow(
+		(new CLabel(_('Name'), 'description'))->setAsteriskMark(),
+		(new CTextBox('description', $data['description'], $readonly))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
+			->setAttribute('autofocus', 'autofocus')
+	)
+	->addRow(
+		new CLabel(_('Operational data'), 'opdata'),
+		(new CTextBox('opdata', $data['opdata'], $readonly))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	);
 
 if ($discovered_trigger) {
 	$triggersFormList->addVar('priority', (int) $data['priority']);
@@ -118,6 +127,7 @@ $expression_row = [
 		$data['expression_field_value'],
 		['readonly' => $data['expression_field_readonly']]
 	))
+		->addClass(ZBX_STYLE_MONOSPACE_FONT)
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -306,6 +316,7 @@ $popup_options = [
 	'dstfrm' => $triggersForm->getName(),
 	'dstfld1' => $data['recovery_expression_field_name']
 ];
+
 if ($data['groupid'] && $data['hostid']) {
 	$popup_options['groupid'] = $data['groupid'];
 	$popup_options['hostid'] = $data['hostid'];
@@ -317,6 +328,7 @@ $recovery_expression_row = [
 		$data['recovery_expression_field_value'],
 		['readonly' => $data['recovery_expression_field_readonly']]
 	))
+		->addClass(ZBX_STYLE_MONOSPACE_FONT)
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -511,52 +523,13 @@ $triggersFormList
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired(),
 		'correlation_tag_row'
+	)
+	->addRow(_('Allow manual close'),
+		(new CCheckBox('manual_close'))
+			->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
+			->setEnabled(!$readonly)
 	);
 
-// Append tags to form list.
-$tags_table = (new CTable())->setId('tbl_tags');
-
-foreach ($data['tags'] as $tag_key => $tag) {
-	$tags = [
-		(new CTextBox('tags['.$tag_key.'][tag]', $tag['tag'], $discovered_trigger, 255))
-			->setWidth(ZBX_TEXTAREA_TAG_WIDTH)
-			->setAttribute('placeholder', _('tag')),
-		(new CTextBox('tags['.$tag_key.'][value]', $tag['value'], $discovered_trigger, 255))
-			->setWidth(ZBX_TEXTAREA_TAG_WIDTH)
-			->setAttribute('placeholder', _('value'))
-	];
-
-	if (!$discovered_trigger) {
-		$tags[] = (new CCol(
-			(new CButton('tags['.$tag_key.'][remove]', _('Remove')))
-				->addClass(ZBX_STYLE_BTN_LINK)
-				->addClass('element-table-remove')
-		))->addClass(ZBX_STYLE_NOWRAP);
-	}
-
-	$tags_table->addRow($tags, 'form_row');
-}
-
-if (!$discovered_trigger) {
-	$tags_table->setFooter(new CCol(
-		(new CButton('tag_add', _('Add')))
-			->addClass(ZBX_STYLE_BTN_LINK)
-			->addClass('element-table-add')
-	));
-}
-
-$triggersFormList->addRow(_('Tags'),
-	(new CDiv($tags_table))
-		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
-);
-
-$triggersFormList->addRow(_('Allow manual close'),
-	(new CCheckBox('manual_close'))
-		->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
-		->setEnabled(!$readonly)
-);
-
 // Append status to form list.
 if (empty($data['triggerid']) && empty($data['form_refresh'])) {
 	$status = true;
@@ -570,6 +543,7 @@ $triggersFormList
 	->addRow(_('Description'),
 		(new CTextArea('comments', $data['comments']))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setMaxlength(DB::getFieldLength('triggers', 'comments'))
 			->setReadonly($discovered_trigger)
 	)
 	->addRow(_('Enabled'), (new CCheckBox('status'))->setChecked($status));
@@ -581,6 +555,17 @@ if (!$data['form_refresh']) {
 }
 $triggersTab->addTab('triggersTab', _('Trigger'), $triggersFormList);
 
+/*
+ * Tags tab
+ */
+$tags_view = new CView('configuration.tags.tab', [
+	'source' => 'trigger',
+	'tags' => $data['tags'],
+	'show_inherited_tags' => $data['show_inherited_tags'],
+	'readonly' => $discovered_trigger
+]);
+$triggersTab->addTab('tags-tab', _('Tags'), $tags_view->render());
+
 /*
  * Dependencies tab
  */
@@ -633,7 +618,7 @@ $dependenciesFormList->addRow(_('Dependencies'),
 				->addClass(ZBX_STYLE_BTN_LINK)
 	]))
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
 );
 $triggersTab->addTab('dependenciesTab', _('Dependencies'), $dependenciesFormList);
 
@@ -644,14 +629,14 @@ if (!empty($data['triggerid'])) {
 			new CSubmit('clone', _('Clone')),
 			(new CButtonDelete(_('Delete trigger?'), url_params(['form', 'hostid', 'triggerid'])))
 				->setEnabled(!$data['limited']),
-			new CButtonCancel(url_param('hostid'))
+			new CButtonCancel()
 		]
 	));
 }
 else {
 	$triggersTab->setFooter(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel(url_param('hostid'))]
+		[new CButtonCancel()]
 	));
 }
 
