@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 $widget = (new CWidget())->setTitle(_('Screens').': '.$data['screen']['name']);
 
-if ($data['screen']['templateid']) {
+if (array_key_exists('templateid', $data['screen'])) {
 	$widget->addItem(get_header_host_table('screens', $data['screen']['templateid']));
 }
 
